unit Marksmain;

interface

uses
  Winapi.Windows, Winapi.Messages, PsAPI, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, VclTee.TeeGDIPlus, Vcl.StdCtrls, Math, Types,
  VCLTee.TeeTools, VCLTee.TeEngine, VCLTee.TeeLegendScrollBar, VCLTee.TeeProcs,
  VCLTee.Chart, Vcl.ExtCtrls, VCLTee.Series;

type
  TForm1 = class(TForm)
    Panel1: TPanel;
    Panel2: TPanel;
    SpectraChart: TChart;
    LegendScrollbar: TLegendScrollBar;
    DragMarksTool: TDragMarksTool;
    NearestTool: TNearestTool;
    Button1: TButton;
    Button2: TButton;
    Button3: TButton;
    Label1: TLabel;
    procedure Button1Click(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure Button3Click(Sender: TObject);
  private
    { Private declarations }
    procedure UpdateMemoryUse;
    procedure SeriesAfterDrawValues(Sender: TObject);
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

uses Generics.Collections;

type
  TMyMarksItem=class(TMarksItem)
  public
    ValueIndex: Integer;
  end;

  TMyMarksItems=class(TObjectList<TMyMarksItem>);

  TMySeriesMarks=class(TObjectList<TMyMarksItems>);

var seriesMarks: TMySeriesMarks;
    visibleMarks: Integer;

function CurrentMemoryUsage: Cardinal;
var
   pmc: TProcessMemoryCounters;
begin
   pmc.cb := SizeOf(pmc) ;
   if GetProcessMemoryInfo(GetCurrentProcess, @pmc, SizeOf(pmc)) then
     Result := pmc.WorkingSetSize
   else
     RaiseLastOSError;
end;

procedure TForm1.Button1Click(Sender: TObject);
var
  i : Integer;
  Series1: TFastLineSeries;
begin
  Series1 := TFastLineSeries.Create(Self);
  Series1.XValues.Order := loNone;
  Series1.DrawAllPoints:=False;
  Series1.DrawAllPointsStyle:=daMinMax;
  for i := 1 to 65000 do
  begin
    Series1.AddXY(i, sin(i/1000+SpectraChart.SeriesCount), '');
  end;
  SpectraChart.AddSeries(Series1);
  SpectraChart.LeftAxis.MaximumOffset:=100;
  SpectraChart.LeftAxis.MinimumOffset:=20;
  Series1.AfterDrawValues:=SeriesAfterDrawValues;
  UpdateMemoryUse;
end;

procedure TForm1.Button2Click(Sender: TObject);
var
  i,j:Integer;
begin
  Screen.Cursor:=crHourglass;
  seriesMarks.Clear;
  for j:=0 to SpectraChart.SeriesCount-1 do
  begin
    seriesMarks.Add(TMyMarksItems.Create);
    visibleMarks:=0;
    SpectraChart.Series[j].CalcVisiblePoints:=false;     // otherwise most marks won't be shown
    with SpectraChart.Series[j].Marks do
    begin
      {BackColor:=SpectraChart.Color;
      Transparent:=false;
      Frame.Color:=SpectraChart.Color;
      Font.Color:=clBlack;
      Automatic.Resize:=true;
      Automatic.Wrap:=true;
      Automatic.Rotate:=true;
      Automatic.Hide:=true;
      Angle:=90;
      ArrowLength:=round(SpectraChart.Height/20);
      Arrow.Color:= SpectraChart.Series[j].Color;
      Arrow.Visible:=true;
      Arrow.Width:=2;
      OnTop:=true;
      Frame.Color:=clnone;
      Shadow.Visible:=false;
      Clip:=true;
      SoftClip:=false;
      ClipText:=false;
      Autoposition:=false;
      Callout.Transparency:=30;
      Callout.Distance:=5;
      Visible:=true;}
      for i:=1 to 65000 do
      begin
        if (CompareValue(sin(i/1000+j), 1, 0.000005)=EqualsValue) then // show marks at maxima as "peak label"
        begin
          seriesMarks[j].Add(TMyMarksItem.Create(SpectraChart));
          seriesMarks[j].Last.ValueIndex:=i;
          seriesMarks[j].Last.Text.Text:=IntToStr(i);
          Inc(visibleMarks);
        end;
          {Item[i].Text.Text:=IntToStr(i)
        else
          Item[i].Text.Text:='';                         //hide undesired marks. Here the memory increase happens!!}
      end;
    end;
  end;
  Screen.Cursor:=crDefault;
  UpdateMemoryUse;
end;

procedure TForm1.Button3Click(Sender: TObject);
var
  i: Integer;
begin
  for i:=SpectraChart.SeriesCount-1 downto 0 do
  begin
    Spectrachart.Series[i].Free;
  end;
  UpdateMemoryUse;
end;

procedure TForm1.FormActivate(Sender: TObject);
begin
    seriesMarks:=TMySeriesMarks.Create;

    with SpectraChart do
    begin
      Tabstop:=true;
      ClipPoints:=True;
      Gradient.Visible:=False;
      Color:=clWhite;
      Walls.Back.Visible:=False;
      Title.Visible := False;
      Legend.Visible := true;
      Legend.Transparent:=false;
      View3D := False;
      Panning.MouseWheel:=pmwNone;
      Legend.ResizeChart:=false;
      Legend.LegendStyle:=lsSeries;
      Axes.FastCalc := True;
      LeftAxis.Axis.Width:=1;
      BottomAxis.Axis.Width:=1;
      LeftAxis.Grid.Visible:=true;
      BottomAxis.Grid.Visible:=true;
      LeftAxis.RoundFirstLabel:=True;
      BottomAxis.RoundFirstLabel:=True;
      LeftAxis.LabelStyle:=talValue;
      BottomAxis.LabelStyle:=talValue;
      LeftAxis.AxisValuesFormat:='#,##0.#######';
      //BottomAxis.AxisValuesFormat:='#,##0.#######';
      LeftAxis.MaximumOffset:=20;
      Axes.Reset;
      LegendScrollbar.Horizontal:=false;
      LegendScrollbar.Bevel:=bvNone;
      LegendScrollbar.DrawStyle:=dsWhenNeeded;
    end;
    UpdateMemoryUse;
end;

procedure TForm1.FormShow(Sender: TObject);
begin
  UpdateMemoryUse;
end;

procedure TForm1.UpdateMemoryUse;
var j: Integer;
    nMarks: Integer;
begin
  Label1.Caption:=FormatFloat('memory use: ,'+FormatSettings.DecimalSeparator+'# K', CurrentMemoryUsage / 1024);

  nMarks:=0;
  for j:=0 to SpectraChart.SeriesCount-1 do
    Inc(nMarks,SpectraChart.Series[j].Marks.Items.Count);

  Label1.Caption:=Label1.Caption+', number of series: '+IntToStr(SpectraChart.SeriesCount)+
                  ', number of marks: ' + FormatFloat(','+FormatSettings.DecimalSeparator+'#', visibleMarks);
end;

procedure TForm1.SeriesAfterDrawValues(Sender: TObject);
var i, j: Integer;
begin
  if Sender is TFastLineSeries then
  with TFastLineSeries(Sender) do
  begin
    for i:=0 to seriesMarks.Count-1 do
      for j:=0 to seriesMarks[i].Count-1 do
        with seriesMarks[i][j] do
        begin
          ShapeBounds.Left:=CalcXPos(ValueIndex);
          ShapeBounds.Top:=CalcYPos(ValueIndex);
          CalcBounds(SpectraChart);
          ShapeBounds.SetLocation(ShapeBounds.Left - (ShapeBounds.Width div 2), ShapeBounds.Top - ShapeBounds.Height - round(SpectraChart.Height/20));        
          seriesMarks[i][j].Draw;
        end;
  end;
end;

end.



